import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/extensions/extensions.dart';

class Sort {
  Sort._();

  static Map<String, bool?> getSorts(String? sortsString) {
    if (sortsString == null || sortsString.isEmpty) return {};

    Map<String, bool?> map = {};

    for (var e in sortsString.split(';')) {
      String field = e.split(':').first;
      String sortString = e.split(':').last;

      bool? sort;

      if (sortString == '1') {
        sort = true;
      } else if (sortString == '-1') {
        sort = false;
      }

      if (field.isNotEmpty && sortString.isNotEmpty && field != sortString) {
        map[field] = sort;
      }
    }
    return map;
  }

  static String toSortString(Map<String, bool?> sorts) {
    return sorts
        .where((field, sort) => field.isNotEmpty && sort != null)
        .entries
        .map(
      (e) {
        String sortString = e.value! ? '1' : '-1';
        return '${e.key}:$sortString';
      },
    ).join(';');
  }
}

mixin SortMixin on BaseController {
  final Map<String, bool?> sorts = {};

  /// Thêm [sorts] vào [params]
  ///
  /// Ví dụ:
  /// ```dart
  /// @override
  /// Future<Map<String, List<Map<String, Object?>?>?>?>? loadData() async {
  ///   addSort();
  ///   var data = await SQLService.execute(
  ///     'sp_...',
  ///     params: params,
  ///     formatJson: Parameter.FORMAT_JSON,
  ///   );
  ///   return data;
  /// }
  /// ```
  void addSort() {
    sorts.forEach(
      (key, value) {
        if (key.isNotEmpty && value != null) {
          params[key] = value ? 1 : -1;
        }
      },
    );
  }

  /// Áp dụng sắp xếp => cập nhật dữ liệu => cập nhật giao diện
  ///
  /// Hỗ trợ kiểu
  /// ```dart
  /// String?
  /// Map<String, bool?>
  /// ```
  ///
  /// Ví dụ
  /// ```dart
  /// applySort('sortGia:1') // từ thấp tới cao
  /// applySort('sortGia:-1') // từ cao tới thấp
  /// // hoặc
  /// applySort({
  ///   'sortGia': true, // từ thấp tới cao
  /// })
  /// applySort({
  ///   'sortGia': false, // từ cao tới thấp
  /// })
  /// ```
  void applySort(dynamic value) {
    if (value == null) return;

    if (value is String?) {
      sorts.clear();
      sorts.addAll(Sort.getSorts(value));
      reload();
    } else if (value is Map<String, bool?>) {
      sorts.clear();
      sorts.addAll(value);
      reload();
    }
  }
}
